;********************************************************************************
;  * @file    lcd_1602.s														*
;  * @author  PLUM, Maciej Szumski												*
;  * @version V1.0																*
;  * @date    12-February-2016													*
;  * @brief   This file contains all functions for the LCD1602					*
;  ******************************************************************************

;------------------- Definicje portow GPIOC -------------------
GPIOC_base	EQU	0x40011000	; port C
GPIOE_base	EQU	0x40011800	; port E
GPIOx_CRL	EQU	0x00		; port Configuration Register Low
GPIOx_CRH	EQU	0x04		; port Configuration Register High
GPIOx_IDR	EQU	0x08		; port Input Data Register
GPIOx_ODR	EQU	0x0C		; port Output Data Register
GPIOx_BSRR	EQU	0x10		; port Bit Set/Reset Register
GPIOx_BRR	EQU	0x14		; port Bit Reset Register
	
;------------------ Definicje pinow LCD1602 -------------------	
LCD_RS		EQU	0x1000		; 1:DATA, 0:INSTRUCTION REGISTER
LCD_RW		EQU	0x0800		; 1:READ, 0:WRITE
LCD_E		EQU	0x0400		; 1:ENABLE
LCD_b4b7	EQU 0x000F		; bity danych b4-b5-b6-b7


			AREA lcd1602asm, CODE, READONLY

;-------------------------- delay_us --------------------------
; opoznienie programowe, counter = 8'000'000 to 1 sekunda
; void delay(unsigned int);
;INPUT:  r0 = unsigned int counter (niszczy r0)
;OUTPUT: void
; counter =      38 <->   5 us (mikrosekund)
;		  =      78 <->  10 us
;		  =     784 <-> 100 us
;		  =  32'800 <->   4.1 ms
;		  = 120'000 <->  15 ms
;         =     8e6 <->   1 sekunda
			EXPORT delay_us
			ALIGN
delay_us	PROC
			B    label02
label01		NOP
			NOP
label02		CMP  r0,#0 ; czy zero na starcie ?
			SUB  r0,r0,#1
			BNE  label01
			BX   lr
			ENDP

;------------------------ LCD_PutNibble -----------------------
; zapisuje bity odwrocone b0-b1-b2-b3
; void LCD_PutNibble(unsigned char);
			ALIGN
LCD_PutNibble	PROC
			PUSH  {r1-r3,lr}			; 4 rejestry na stos (PRESERVE8)
			LDR   r2,=GPIOC_base		; port GPIOC
			MOV   r1,#LCD_E				; bit ENABLE
			STR   r1,[r2,#GPIOx_BSRR]	; ustaw bit ENABLE = 1
			RBIT  r0,r0					; odwrocenie kolejnosci bitow
			LSR	  r0,r0,#28				; przesuwa w prawo, wchodza zera
			STR   r0,[r2,#GPIOx_BSRR]	; ustawia jedynki w danych
			MVN   r0,r0					; neguje bit po bicie
			LSL   r0,r0,#28				; przesuwa w lewo
			LSR	  r0,r0,#28				; przesuwa w prawo, wchodza zera
			STR   r0,[r2,#GPIOx_BRR]	; zeruje zera w danych
			MOVS  r0,#1
			BL    delay_us				; opoznienie minimum 230 ns
			STR   r1,[r2,#GPIOx_BRR]	; wyzeruj bit ENABLE = 0
			POP   {r1-r3,pc}
			ENDP

;----------------------- LCD_ReadStatus  ----------------------
; unsigned char LCD_ReadStatus(void)
			ALIGN
LCD_ReadStatus	PROC
			PUSH  {r1-r3,lr}			; 4 rejestry na stos (PRESERVE8)
			LDR   r2,=GPIOC_base	 	; port GPIOC
			; Configuration Register Low
			LDR   r1,=0xFFFF0000	 	; maska AND
			LDR	  r0,[r2,#GPIOx_CRL]	
			AND	  r0,r0,r1
			MOVW  r1,#0x4444		 	; maska OR
			ORR   r0,r0,r1
			STR	  r0,[r2,#GPIOx_CRL] 	; input floating
			; RS=0, RW=1
			MOV   r1,#LCD_RS		 	; bit RS
			STR   r1,[r2,#GPIOx_BRR] 	; wyzeruj bit RS = 0
			MOV   r1,#LCD_RW		 	; bit R/W
			STR   r1,[r2,#GPIOx_BSRR]	; ustaw bit R/W = 1
			NOP
			NOP						 	; czekamy 40ns
			MOV   r1,#LCD_E			 	; bit ENABLE
			STR   r1,[r2,#GPIOx_BSRR]	; ustaw bit ENABLE = 1
			MOVS  r0,#1
			BL    delay_us				; opoznienie minimum 230 ns
			LDR	  r3,[r2,#GPIOx_IDR]	; port Input Data Register
			AND	  r3,r3,#0x0F
			RBIT  r3,r3					; odwrocenie kolejnosci bitow
			LSR	  r3,r3,#24				; przesuwa w prawo, wchodza zera
			STR   r1,[r2,#GPIOx_BRR]	; wyzeruj bit ENABLE = 0
			MOVS  r0,#1
			BL    delay_us				; opoznienie minimum 270 ns
			STR   r1,[r2,#GPIOx_BSRR]	; ustaw bit ENABLE = 1
			MOVS  r0,#1
			BL    delay_us				; opoznienie minimum 230 ns
			LDR	  r0,[r2,#GPIOx_IDR]	; port Input Data Register
			RBIT  r0,r0					; odwrocenie kolejnosci bitow
			LSR	  r0,r0,#28				; przesuwa w prawo, wchodza zera
			ORR	  r0,r0,r3				; r0 = 000000FF
			STR   r1,[r2,#GPIOx_BRR]	; wyzeruj bit ENABLE = 0
			; Configuration Register Low
			LDR   r1,=0xFFFF0000		; maska AND
			LDR	  r3,[r2,#GPIOx_CRL]	
			AND	  r3,r3,r1
			MOVW  r1,#0x1111			; maska OR
			ORR   r3,r3,r1
			STR	  r3,[r2,#GPIOx_CRL]
			POP   {r1-r3,pc}			; r0 = odczytany bajt
			ENDP
			
;---------------------- LCD_SendCommand  ----------------------
;void LCD_SendCommand(unsigned char)
			EXPORT LCD_SendCommand
			ALIGN
LCD_SendCommand	PROC
			PUSH  {r1-r3,lr}			; 4 rejestry na stos (PRESERVE8)
			MOV   r3,r0
label03		BL    LCD_ReadStatus
			ANDS  r0,r0,#0x80
			BNE   label03	; czekamy na BUSY = 0
			LDR   r2,=GPIOC_base	 	; port GPIOC
			MOV   r1,#LCD_RS:OR:LCD_RW	; bity RS, R/W
			STR   r1,[r2,#GPIOx_BRR]	; wyzeruj RS=0, R/W=0
			LSR	  r0,r3,#4				; starszy nibble
			BL    LCD_PutNibble
			MOV   r0,r3
			BL    LCD_PutNibble
			POP   {r1-r3,pc}
			ENDP
	
;--------------------- LCD_WriteCharacter ---------------------
;void LCD_WriteCharacter(unsigned char)
			EXPORT LCD_WriteCharacter
			ALIGN
LCD_WriteCharacter	PROC
			PUSH  {r1-r3,lr}			; 4 rejestry na stos (PRESERVE8)
			MOV   r3,r0
label04		BL    LCD_ReadStatus
			ANDS  r0,r0,#0x80
			BNE   label04	; czekamy na BUSY = 0
			LDR   r2,=GPIOC_base	 	; port GPIOC
			MOV   r1,#LCD_RS
			STR   r1,[r2,#GPIOx_BSRR]	; ustaw RS = 1
			MOV   r1,#LCD_RW
			STR   r1,[r2,#GPIOx_BRR]	; wyzeruj R/W = 0
			LSR	  r0,r3,#4				; starszy nibble
			BL    LCD_PutNibble
			MOV   r0,r3
			BL    LCD_PutNibble
			POP   {r1-r3,pc}
			ENDP
			
;------------------------- nibble2Hex -------------------------
;unsigned char nibble2Hex(unsigned int)
			ALIGN
nibble2Hex	PROC
			AND   r0,r0,#0x0F
			CMP   r0,#10
			BCS   label05			; unsigned Higher or Same
			ADD   r0,r0,#0x30
			UXTB  r0,r0
			BX    lr
label05		ADD   r0,r0,#0x37
			UXTB  r0,r0
			BX    lr
			ENDP
				
;------------------------ LCD_WriteHex ------------------------
; void LCD_WriteHex(unsigned int var);
			EXPORT LCD_WriteHex
			ALIGN
LCD_WriteHex	PROC
			PUSH  {r1-r3,lr}	; 4 rejestry na stos (PRESERVE8)
			MOV   r1,r0
			MOVS  r2,#8 		; licznik znakow
label06		ROR   r1,r1,#28
			AND   r0,r1,#0x0F
			BL    nibble2Hex
			BL    LCD_WriteCharacter
			SBCS  r2,r2,#1
			BNE   label06
			BX    lr
			ENDP

;------------------------ LCD_Initialize ------------------------
; programowa inicjalizacja wyswietlacza LCD1602
; inicjuje tez odpowiednie linie portu GPIOC
; void LCD_Initialize(void);
;INPUT:  void
;OUTPUT: void
			EXPORT LCD_Initialize
			IMPORT LCD_SetUserChar
			ALIGN
LCD_Initialize	PROC
			PUSH  {r1-r3,lr}		; 4 rejestry na stos (PRESERVE8)
			; zaczynam od zaprogramowania portu GPIOC
			; 10MHz, output Push-Pull <--> 0x1
			; bity: CRH <-> 1C, CRL <-> 0F
			LDR   r2,=GPIOC_base	; port GPIOC
			; Configuration Register Low
			LDR   r1,=0xFFFF0000	; maska AND
			LDR	  r0,[r2,#GPIOx_CRL]	
			AND	  r0,r0,r1
			MOVW  r1,#0x1111		; maska OR
			ORR   r0,r0,r1
			STR	  r0,[r2,#GPIOx_CRL]
			; Configuration Register High
			LDR   r1,=0xFFF000FF	; maska AND
			LDR	  r0,[r2,#GPIOx_CRH]	
			AND	  r0,r0,r1
			LDR   r1,=0x00011100	; maska OR
			ORR   r0,r0,r1
			STR	  r0,[r2,#GPIOx_CRH]
			; wyzerujemy sygnaly wyswietlacza
			MOVW  r1,#LCD_RS:OR:LCD_RW:OR:LCD_E:OR:LCD_b4b7
			STR   r1,[r2,#GPIOx_BRR] ; wyzeruj bity
			; inicjalizacja wyswietlacza
			LDR	  r0,=120000
			BL    delay_us			; opoznienie 15 ms
			; sygnaly wyzerowane: RS = 0 i RW = 0
			MOVS  r0,#0x03			; instrukcja 0x3
			BL    LCD_PutNibble
			MOVW  r0,#32800
			BL    delay_us			; opoznienie 4.1 ms
			MOVS  r0,#0x03			; instrukcja 0x3
			BL    LCD_PutNibble
			MOV	  r0,#784
			BL    delay_us			; opoznienie 100 us
			MOVS  r0,#0x03			; instrukcja 0x3
			BL    LCD_PutNibble
			MOV	  r0,#784
			BL    delay_us			; opoznienie 100 us
			MOVS  r0,#0x02			; instrukcja 0x2
			BL    LCD_PutNibble
			MOV	  r0,#784
			BL    delay_us			; opoznienie 100 us
			MOVS  r0,#0x28			; dwie linie, 5x7
			BL    LCD_SendCommand
			MOVS  r0,#0x08			; Display off
			BL    LCD_SendCommand
			MOVS  r0,#0x01			; Clear Display
			BL    LCD_SendCommand
			MOVS  r0,#0x06			; Entry Mode Set
			BL    LCD_SendCommand
			MOVS  r0,#0x0C			; Display on
			BL    LCD_SendCommand				
			; generator polskich znakow
			MOVS  r0,#0		; od pierwszego
			MOVS  r1,#8
			LDR	  r2,=polskie
			BL    LCD_SetUserChar
			POP   {r1-r3,pc}
			ENDP

			ALIGN
;--------------- generator polskich znakow ----------------
polskie		DCD 	0x100E0402
			DCD 	0x001E010E	; s'
			DCD 	0x010E0000
			DCD 	0x010F110F	; a,
			DCD 	0x021F0004
			DCD 	0x001F0804	; z'
			DCD 	0x100E0402
			DCD 	0x000E1110  ; c'
			DCD 	0x0406040C
			DCD 	0x000E040C	; l'
			DCD 	0x110E0000
			DCD 	0x040E101F	; e,
			DCD 	0x19160402
			DCD 	0x00111111	; n'
			DCD 	0x110E0402
			DCD 	0x000E1111	; o'
				
			END





