/**************************************************************
 *  Systemy Mikroprocesorowe w Sterowaniu,  WEiTI PW 				  *
 *  ZL27ARM: PROJEKT_05 klawiatura									 					*
 **************************************************************/
#include "stm32f10x.h"
#include <stdbool.h> 		// true, false
#include <stdio.h>
#include ".\..\Drivers\KAmodKB4x4\KB_4x4.h"
#include ".\..\Drivers\LCD1602\lcd_1602.h"

// zmienne globalne do zapisywania znakow z klawiatury ----------
unsigned char key_buffer[32] = {0}; 		// deklaracja tablicy
volatile unsigned int key_pointer = 0;	// numer elementu w tablicy 'key_buffer'
unsigned char txt2LCD[32] = {0}; 				// bufor funkcji sprintf
unsigned char KB2mem[32] = {0};   			// bufor funkcji sscanf

int  RCC_Config(void);
void GPIO_Config(void);
void NVIC_Config(void);
int SysTick_Config_Div8(unsigned int Ticks);

int main(void) {
	//------------------- deklaracje zmiennych ------------------
	unsigned int i;
	
  //------------------ konfiguracja systemu -------------------
  RCC_Config();   
  GPIO_Config(); 
  NVIC_Config(); 
	// zgasimy wszystkie diody LED
  GPIO_ResetBits(GPIOB, 0xFF00u);
	LCD_Initialize();              //inicjalizacja wyswietlacza
	LCD_SendCommand(LCD_DISPLAY_ONOFF | LCD_DISPLAY_ON | LCD_CURSOR_ON);	
	SysTick_Config_Div8(9000000ul);
	
	//------------------------ program C ------------------------
	for(i=0; i<32; i++)
		key_buffer[i] = ' ';
	
	while (1) {
		LCD_GoTo(0,0);
		for(i=0; i<16; i++){
			LCD_WriteCharacter(key_buffer[i]);
		}
		LCD_GoTo(0,1);
		for(i=16; i<32; i++){
			LCD_WriteCharacter(key_buffer[i]);
		}
		GPIO_WriteBit(GPIOB, GPIO_Pin_14, Bit_SET);
		delay_us(4000000);
		GPIO_WriteBit(GPIOB, GPIO_Pin_14, Bit_RESET);
		delay_us(4000000);
	}
}

//------------ konfiguracja RCC: sygnaly taktujace --------------------------------------------
int RCC_Config(void) {
  ErrorStatus HSEStartUpStatus;                          // zmienna opisujaca rezultat uruchomienia HSE

  RCC_DeInit();                                          // RCC na 'default reset state'
  RCC_HSEConfig(RCC_HSE_ON);                             // wlacz HSE
  HSEStartUpStatus = RCC_WaitForHSEStartUp();            // czekaj na gotowosc HSE
  if(HSEStartUpStatus == SUCCESS) {
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);//
    FLASH_SetLatency(FLASH_Latency_2);                   // zwloka Flasha: 2 takty dla 72MHz
    RCC_HCLKConfig(RCC_SYSCLK_Div1);                     // HCLK=SYSCLK
    RCC_PCLK2Config(RCC_HCLK_Div1);                      // PCLK2=HCLK
    RCC_PCLK1Config(RCC_HCLK_Div2);                      // PCLK1=HCLK/2
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9); // PLLCLK = HSE*9 czyli 8MHz * 9 = 72 MHz
    RCC_PLLCmd(ENABLE);                                  // wlacz PLL
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);  // czekaj na uruchomienie PLL
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);           // ustaw PLL jako zrodlo sygnalu zegarowego
    while(RCC_GetSYSCLKSource() != 0x08);                // czekaj az PLL bedzie sygnalem zegarowym systemu
// konfiguracja sygnalow taktujacych uzywanych peryferii 
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE); // wlacz taktowanie portu AFIO (przerwania zewnetrzne)
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);// wlacz taktowanie portu GPIO A (przyciski)
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);// wlacz taktowanie portu GPIO B (LEDy)
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);// wlacz taktowanie portu GPIO C (LCD1602)
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOD, ENABLE);// wlacz taktowanie portu GPIO D (klawiatura)
		return 0;
  }
	else
		return 1;
}
//------------------ konfiguracja portow GPIO -------------------------------------------------
void GPIO_Config(void) {
  GPIO_InitTypeDef  GPIO_InitStructure; 
	
	// port A - Przyciski i joystick
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
	
  // port B - diody LED
  GPIO_InitStructure.GPIO_Pin = 0xFF00u; 	// piny 15,14,...9,8
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;      // wyjscie w trybie push-pull
  GPIO_Init(GPIOB, &GPIO_InitStructure);
	
	// port D - klawiatura
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_OD;      // wyjscie w trybie Open-Drain
  GPIO_Init(GPIOD, &GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;	// wejscie w trybie FLOATING,
  GPIO_Init(GPIOD, &GPIO_InitStructure);
	GPIO_ResetBits(GPIOD, GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13);
	GPIO_AFIODeInit();
}

// ----------- konfiguracja kontrolera przerwan NVIC ------------------------------------------
void NVIC_Config(void) {
	NVIC_InitTypeDef  NVIC_InitStructure; 
  EXTI_InitTypeDef  EXTI_InitStructure; 
	
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);	// tablica wektorow przerwan we FLASH
	// Konfiguracja NVIC - ustawienia priorytetow przerwania EXTI0
  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);                //Wybor modelu grupowania przerwan
  // linia 6 = COL1 -------------------------------------------------------------------------
  GPIO_EXTILineConfig(GPIO_PortSourceGPIOD, GPIO_PinSource6);    //Ustawienie zrodla przerwania
  //Konfiguracja przerwania EXTI na linie 6
  EXTI_InitStructure.EXTI_Line = EXTI_Line6;                     //Wybor linii 
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;            //Ustawienie generowania przerwania (a nie zdarzenia)
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;        //Wyzwalanie zboczem opadajacym (wcisniecie przycisku)
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;                      //Wlaczenie przerwania
  EXTI_Init(&EXTI_InitStructure);
	
	// linia 7 = COL2 -------------------------------------------------------------------------
  GPIO_EXTILineConfig(GPIO_PortSourceGPIOD, GPIO_PinSource7);    //Ustawienie zrodla przerwania
  //Konfiguracja przerwania EXTI na linie 7
  EXTI_InitStructure.EXTI_Line = EXTI_Line7;                     //Wybor linii 
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;            //Ustawienie generowania przerwania (a nie zdarzenia)
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;        //Wyzwalanie zboczem opadajacym (wcisniecie przycisku)
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;                      //Wlaczenie przerwania
  EXTI_Init(&EXTI_InitStructure);
	
	// linia 8 = COL3 ---------------------------------------------------------------------------
  GPIO_EXTILineConfig(GPIO_PortSourceGPIOD, GPIO_PinSource8);    //Ustawienie zrodla przerwania
  //Konfiguracja przerwania EXTI na linie 8
  EXTI_InitStructure.EXTI_Line = EXTI_Line8;                     //Wybor linii 
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;            //Ustawienie generowania przerwania (a nie zdarzenia)
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;        //Wyzwalanie zboczem opadajacym (wcisniecie przycisku)
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;                      //Wlaczenie przerwania
  EXTI_Init(&EXTI_InitStructure);
	
	// linia 9 = COL4 ---------------------------------------------------------------------------
  GPIO_EXTILineConfig(GPIO_PortSourceGPIOD, GPIO_PinSource9);    //Ustawienie zrodla przerwania
  //Konfiguracja przerwania EXTI na linie 9
  EXTI_InitStructure.EXTI_Line = EXTI_Line9;                     //Wybor linii 
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;            //Ustawienie generowania przerwania (a nie zdarzenia)
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;        //Wyzwalanie zboczem opadajacym (wcisniecie przycisku)
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;                      //Wlaczenie przerwania
  EXTI_Init(&EXTI_InitStructure);
	
	// Wybor konfigurowanego IRQ ----------------------------------------------------------------
	NVIC_InitStructure.NVIC_IRQChannel = EXTI9_5_IRQn;             //Wybor konfigurowanego IRQ
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 3;      //Priorytet grupowy
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 3;             //Podpriorytet
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;                //Wlaczenie obslugi IRQ
  NVIC_Init(&NVIC_InitStructure);
}
// --------- konfiguracja licznika systemowego SysTick ----------------------------------------
int SysTick_Config_Div8(unsigned int Ticks){ 
  unsigned int ctrl;

  if (Ticks > SysTick_LOAD_RELOAD_Msk)  return 1;             	// czy wartosc poczatkowa nie przekracza max
                                                                
  SysTick->LOAD = (Ticks & SysTick_LOAD_RELOAD_Msk) - 1;        // ustaw wartosc poczatkowa licznika 
  NVIC_SetPriority (SysTick_IRQn, 0);                           // ustaw priorytet przerwania
  SysTick->VAL  = 0;                                            // ustaw wartosc aktualna licznika
  ctrl=SysTick_CTRL_TICKINT_Msk | SysTick_CTRL_ENABLE_Msk;      // ustaw flagi wlaczenia SysTick IRQ  i samego licznika
	ctrl &= SysTick_CLKSource_HCLK_Div8;
  SysTick->CTRL = ctrl;                  // zapisz ustawienia do rejestru sterujacego SysTick (i wlacz licznik)
  return 0;
}

/**************** (C) COPYRIGHT 2016 PLUMicroelectronics *****END OF FILE******/



