/**************************************************************
 *  Systemy Mikroprocesorowe w Sterowaniu,  WEiTI PW 				  *
 *  ZL27ARM: PROJEKT_25  MODBUS SLAVE													*
 **************************************************************/
#include "stm32f10x.h"
#include <stdio.h>
#include ".\..\Drivers\LCD1602\lcd_1602.h"

int  RCC_Config(void);
void GPIO_Config(void);
void NVIC_Config(void);
void DMA_Config(void);
void ADC_Config(void);
void USART_Config(void);
int SysTick_Config_Div8(unsigned int Ticks);

// zmienne globalne -------------------------------------------
uint8_t Rx_message[32]={0};			// bufor na znaki odebrane
uint8_t Tx_message[32]={0};			// bufor na znaki do nadania
unsigned int Tx_num = 0;				// licznik ramek nadanych
unsigned int Rx_num = 0;				// licznik ramek odebranych
uint32_t Tx_index = 0;					// wskaznik na miejsce w Tx_message
uint32_t Rx_index = 0;					// wskaznik na miejsce w Rx_message
unsigned int SysTick_count = 0;	// licznik do odmierzania przerwy > 3.5T
unsigned char txt2LCD[32] = {0};// bufor funkcji sprintf
uint16_t PC2_adc[100]={0};			// bufor na wartosci binarne z ADC
float current, T;

int main(void) {
	//------------------- deklaracje zmiennych ------------------  float current, T;
	unsigned int i;
	
  //------------------ konfiguracja systemu -------------------
  RCC_Config();   
  GPIO_Config(); 
  NVIC_Config(); 
	USART_Config();
	DMA_Config();
	ADC_Config();
  GPIO_ResetBits(GPIOB, 0xFF00u);// zgasimy wszystkie diody LED
	LCD_Initialize();              //inicjalizacja wyswietlacza
	LCD_SendCommand(LCD_DISPLAY_ONOFF | LCD_DISPLAY_ON | LCD_CURSOR_ON);
	SysTick_Config_Div8(90000ul);	// przerwanie co 0.01 sekundy

	//------------------------ program C ------------------------
	while (1) {
		current =0.0;
		for(i=0; i<100; i++){
			current += (float)PC2_adc[i] * 0.00667904;		// przeliczone na prad [mA]
		}
		current /= 100.0;
		T = 5.625*current - 52.5;
		
		LCD_GoTo(0,0);		// pierwsza linia
		sprintf((char *)txt2LCD,"Rx=%4i,Tx=%4i", Rx_num, Tx_num);
		LCD_WriteText(txt2LCD); 
		LCD_GoTo(0,1);		// druga linia
		sprintf((char *)txt2LCD,"i=%4.2f,T=%4.2f", current, T);
		LCD_WriteText(txt2LCD); 

		delay_us(1000000);
	}
}
//------------ konfiguracja RCC: sygnaly taktujace ------------
int RCC_Config(void) {
  ErrorStatus HSEStartUpStatus;                          // zmienna opisujaca rezultat uruchomienia HSE

  RCC_DeInit();                                          // RCC na 'default reset state'
  RCC_HSEConfig(RCC_HSE_ON);                             // wlacz HSE
  HSEStartUpStatus = RCC_WaitForHSEStartUp();            // czekaj na gotowosc HSE
  if(HSEStartUpStatus == SUCCESS) {
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);//
    FLASH_SetLatency(FLASH_Latency_2);                   // zwloka Flasha: 2 takty dla 72MHz
    RCC_HCLKConfig(RCC_SYSCLK_Div1);                     // HCLK=SYSCLK
    RCC_PCLK2Config(RCC_HCLK_Div1);                      // PCLK2=HCLK
    RCC_PCLK1Config(RCC_HCLK_Div2);                      // PCLK1=HCLK/2
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9); // PLLCLK = HSE*9 czyli 8MHz * 9 = 72 MHz
    RCC_PLLCmd(ENABLE);                                  // wlacz PLL
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);  // czekaj na uruchomienie PLL
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);           // ustaw PLL jako zrodlo sygnalu zegarowego
    while(RCC_GetSYSCLKSource() != 0x08);                // czekaj az PLL bedzie sygnalem zegarowym systemu
		// konfiguracja sygnalow taktujacych uzywanych peryferii 
		RCC_ADCCLKConfig(RCC_PCLK2_Div6);                    // ADCCLK = PCLK2/6 = 72 MHz /6 = 12 MHz
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);// port GPIO A : PA9 (Tx), PA10 (Rx)
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);// GPIO B: LEDy, PB0 = wejscie ADC12_IN8
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);// GPIO C: LCD1602
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE); // wlacz taktowanie ADC1
		RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);   // wlacz taktowanie DMA
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);// wlacz taktowanie USART1
		return 0;
  }
	else
		return 1;
}
//------------------ konfiguracja portow GPIO -----------------
void GPIO_Config(void) {
  GPIO_InitTypeDef  GPIO_InitStructure; 

  // LED1 ... LED8 na plytce
  GPIO_InitStructure.GPIO_Pin = 0xFF00u; 	// piny 15,14,...9,8
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;      // wyjscie w trybie push-pull
  GPIO_Init(GPIOB, &GPIO_InitStructure);
	
	GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_0;   
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;	//	wejscie analogowe z PB0 = ADC12_IN8
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIOB, &GPIO_InitStructure);
	
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;							// PA9 -> Tx
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
 	GPIO_Init(GPIOA, &GPIO_InitStructure);

 	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;						// PA10 <- Rx
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
 	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
 	GPIO_Init(GPIOA, &GPIO_InitStructure);
}
// ----------- konfiguracja kontrolera przerwan NVIC ----------
void NVIC_Config(void) {
	NVIC_InitTypeDef NVIC_InitStructure;
	NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0); 
	
  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);
 	//Przerwanie od USART1
 	NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQn;
 	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
 	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
 	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
 	NVIC_Init(&NVIC_InitStructure);
}
// --------- konfiguracja licznika systemowego SysTick --------
int SysTick_Config_Div8(unsigned int Ticks){ 
  unsigned int ctrl;

  if (Ticks > SysTick_LOAD_RELOAD_Msk)  return 1;             	// czy wartosc poczatkowa nie przekracza max
                                                                
  SysTick->LOAD = (Ticks & SysTick_LOAD_RELOAD_Msk) - 1;        // ustaw wartosc poczatkowa licznika 
  NVIC_SetPriority (SysTick_IRQn, 0);                           // ustaw priorytet przerwania
  SysTick->VAL  = 0;                                            // ustaw wartosc aktualna licznika
  ctrl=SysTick_CTRL_TICKINT_Msk | SysTick_CTRL_ENABLE_Msk;      // ustaw flagi wlaczenia SysTick IRQ  i samego licznika
	ctrl &= SysTick_CLKSource_HCLK_Div8;
  SysTick->CTRL = ctrl;                  // zapisz ustawienia do rejestru sterujacego SysTick (i wlacz licznik)
  return 0;
}
// -------------- konfiguracja przetwornika ADC1 --------------
void ADC_Config(void){ //konfigurowanie przetwornika AC
  ADC_InitTypeDef ADC_InitStructure;
	  
	ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;                  // jeden przetwornik, praca niezalezna
	ADC_InitStructure.ADC_ScanConvMode = DISABLE;                       // pomiar jednego kanalu, skanowanie kanalow nie potrzebne
  ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;               	// pomiar w trybie jednokrotnym
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;	// brak wyzwalania zewnetrznego
	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;            	// wyrownanie danych do prawej - 12 mlodszych bitow znaczacych
	ADC_InitStructure.ADC_NbrOfChannel = 1; 	                          // liczba uzywanych kanalow =1
	ADC_Init(ADC1, &ADC_InitStructure);                                 // incjalizacja przetwornika

	ADC_RegularChannelConfig(ADC1, ADC_Channel_8, 1, ADC_SampleTime_1Cycles5); 
                                                                      // kanal 8 - PB0 = wejscie ADC12_IN8
                                                                      // grupa podstawowa, czas probkowania 1,5 + 12,5 = 14 cykli
	ADC_Cmd(ADC1, ENABLE);                                              // wlacz ADC1

	ADC_ResetCalibration(ADC1);                                         // reset rejestrow kalibracyjnych ADC1
	while(ADC_GetResetCalibrationStatus(ADC1));                         // czekamy na wykonanie resetu
	ADC_StartCalibration(ADC1);                                         // kalibracja ADC1
	while(ADC_GetCalibrationStatus(ADC1));                              // czekamy na zakonczenie kalibracji ADC1
	
	ADC_DMACmd(ADC1,ENABLE); 	                                          // wlaczenie DMA dla ADC1
}
// ----------------- konfiguracja kanalu DMA ------------------
void DMA_Config(void){
  #define ADC1_DR_Address 0x4001244C;                                             //adres rejestru ADC1->DR
  DMA_InitTypeDef DMA_InitStructure;
	
  DMA_DeInit(DMA1_Channel1);                                                  // usun ewentualna poprzednia konfiguracje DMA
  DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)ADC1_DR_Address;  			// adres docelowy transferu
  DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t) PC2_adc;            			// adres poczatku bloku do przeslania
  DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;                          // kierunek transferu
  DMA_InitStructure.DMA_BufferSize = 100;                                     // liczba elementow do przeslania (dlugosc bufora)
  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;            // wylaczenie autoinkrementacji adresu po stronie ADC
  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;                     // wlaczenie autoinkrementacji adresu po stronie pamieci (bufora)
  DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;	// rozmiar pojedynczych przesylanych danych po stronie ADC (HalfWord = 16bit)
  DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;         // rozmiar pojedynczych przesylanych danych po stronie pamieci (bufora) = 16bit
  DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;                             // tryb dzialania kontrolera DMA - powtarzanie cykliczne
  DMA_InitStructure.DMA_Priority = DMA_Priority_High;                         // priorytet DMA - wysoki
  DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;                                // wylaczenie obslugi transferu z pamieci do pamieci
  DMA_Init(DMA1_Channel1, &DMA_InitStructure);                                // zapis konfiguracji
  DMA_Cmd(DMA1_Channel1, ENABLE);		//Wlacz DMA, kanal 1
}
// ------------------- konfiguracja USART1 --------------------
void USART_Config(void){
  USART_InitTypeDef  USART_InitStructure; 

  USART_InitStructure.USART_BaudRate = 9600;                                      // predkosc transmisji =9600bps
  USART_InitStructure.USART_WordLength = USART_WordLength_9b;                     // 8 bitow danych + PARITY
  USART_InitStructure.USART_StopBits = USART_StopBits_1;                          // 1 bit stopu
  USART_InitStructure.USART_Parity = USART_Parity_Even;                           // parzystosc
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None; // bez kontroli przeplywu danych
  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;	                // nadawanie i odbior
	USART_Init(USART1, &USART_InitStructure);
	
	USART_ITConfig(USART1, USART_IT_RXNE, ENABLE);		// zezwolenie na przerwania od USART Receiver Not Empty
	USART_Cmd(USART1, ENABLE);
}
/**************** (C) COPYRIGHT 2016 PLUMicroelectronics *****END OF FILE******/



