/********************************************
 * Programy asemblerowe.h   30-08-2016   		*
 ********************************************/
#include "stm32f10x.h"

#define MAX_TASKS 3
#define TASK_FLAG_INIT 0x01 // jedynka oznacza zainicjowanie kontekstu
#define TASK_FLAG_EXEC 0x04	// jedynka oznacza brak wczesniejszej egzekucji

// struktury --------------------------------------------------
typedef struct { 		// rozmieszczenie rejestrow na stosie
	uint32_t r4;			// pushed by software
	uint32_t r5;
	uint32_t r6;
	uint32_t r7;
	uint32_t r8;
	uint32_t r9;
	uint32_t r10;
	uint32_t r11;
	uint32_t r0;			// pushed by hardware
	uint32_t r1;
	uint32_t r2;
	uint32_t r3;
	uint32_t r12;
	uint32_t lr;
	uint32_t pc;
	uint32_t psr;
}stack_frame_t;

typedef struct{								// structure of task_table
	uint32_t sp; 	// Task stack pointer
	int flags; 		// Task status flags
} task_table_t;

//--------- Read main stack pointer ---------
//INPUT: 	void
//OUTPUT: r0 = MSP
unsigned int get_MSP(void);

//--------- Write main stack pointer --------
//INPUT:  r0 = MSP
//OUTPUT: void
void set_MSP(unsigned int);

//-------- Read process stack pointer -------
//INPUT:  void
//OUTPUT: r0 = PSP
unsigned int get_PSP(void);
				
//------- Write process stack pointer -------
//INPUT:  r0 = PSP
//OUTPUT: void
void set_PSP(unsigned int);

//--- odczytaj adres powrotu ze stosu MSP ----
// odczytuje adres instrukcji, ktora wywolala
// 		  Hard Fault exception
//INPUT:  void
//OUTPUT: r0 = return address (MSP)
unsigned int get_return_addr(void);

//------- obsluga przerwania SysTick --------
void SysTick_Handler(void);

//- Report HardFault when things goes wrong -
void HardFault_Handler(void);





